unit gui;

{$mode objfpc}{$H+}
{$modeswitch nestedprocvars}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, Menus, ComCtrls,
  ExtCtrls, xmlUtil, guitabbase;

type

  { TFormMain }

  TFormMain = class(TForm)
    MainMenu1: TMainMenu;
    MenuItem1: TMenuItem;
    MenuItem2: TMenuItem;
    MenuItem3: TMenuItem;
    MenuItem4: TMenuItem;
    MenuItem5: TMenuItem;
    MenuItem6: TMenuItem;
    MenuItem7: TMenuItem;
    MenuItem8: TMenuItem;
    MenuItem9: TMenuItem;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    TabControl1: TTabControl;
    procedure addFunction(Sender: TObject); overload;
    procedure deleteFunction(Sender: TObject);
    procedure closeClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure new(Sender: TObject);
    procedure open(Sender: TObject);
    procedure save(Sender: TObject);
    procedure selectTab(Sender: TObject);
  private
    path : String;
    document : XML.TDocument;
    tabs : Array of TGuiTab;
    procedure clear(_destroy:boolean=false);
    procedure addFunction(xmlEl:XML.TElement); overload;
  public

  end;

var
  FormMain: TFormMain;

implementation

{$R *.lfm}

{ TFormMain }

procedure TFormMain.FormCreate(Sender: TObject);
var
  i : integer;
  m : TMenuItem;
begin
  tabs := nil;
  for i := 0 to TGuiTab.availlableCount-1 do begin
    m := TMenuItem.Create(self);
    m.Caption := TGuiTab.availlable[i].name;
    m.Hint := TGuiTab.availlable[i].desc;
    MenuItem7.Add(m);
    m.onClick := @addFunction;
  end;
  document := nil;
  if (paramStr(1) <> '') then begin
    path := paramStr(1);
    open(sender);
  end else clear;
end;

procedure TFormMain.clear(_destroy:boolean=false);
var i : integer;
begin
  for i := 0 to length(tabs)-1 do Application.ReleaseComponent(tabs[i]);
  setLength(tabs, 0);
  TabControl1.Tabs.Clear;
  document.free;
  path := '';
  if (not _destroy) then begin
    document := XML.TDocument.Create;
    document.markUnchanged;
  end;
end;

procedure TFormMain.new(Sender: TObject);
begin
  if (document <> nil) and (document.changed) then
    case (MessageDlg('Changes', 'Do you wish to save changes on current file?', mtConfirmation, [mbYes, mbNo, mbAbort], 0)) of
     mrYes : save(sender);
     mrAbort : exit;
    end;
  clear;
end;

procedure TFormMain.open(Sender: TObject); // TODO: Opening a document if an other non-empty document is already open causes memory leaks.
var
  str : TStringList;
  pathBackup : string;
  el : XML.TElement;
  i : integer;
begin
  pathBackup := '';
  if (document <> nil) and (document.changed) then begin
    case (MessageDlg('Changes', 'Do you wish to save changes on current file?', mtConfirmation, [mbYes, mbNo, mbAbort], 0)) of
     mrYes : save(sender);
     mrAbort : exit;
    end;
  end else pathBackup := path;
  clear;
  if (pathBackup <> '') then path := pathBackup;
  if (path = '') or (Sender = MenuItem2) then begin
    if (not openDialog1.Execute) then exit;
    path := openDialog1.fileName;
  end;
  str := TStringList.Create;
  try
    str.LoadFromFile(path);
    document.FromStrings(str);
  except
    ShowMessage('Cannot read file "'+path+'"!');
  end;
  str.destroy;
  el := document.getElementsByTagName('GUITABS', 0);
  if (el = nil) then begin
    el := XML.TElement.Create;
    document.appendChild(el);
  end;
  for i := 0 to el.childNodes.count-1 do
    if (el.childNodes[i].nodeType = ELEMENT_NODE) then
      addFunction(XML.TElement(el.childNodes[i]));;
  document.markUnchanged;
end;

procedure TFormMain.save(Sender: TObject);
var str : TStrings;
begin
  if (path = '') or (Sender = MenuItem4) then begin
    if (not saveDialog1.Execute) then exit;
    path := saveDialog1.fileName;
  end;
  str := document.toStrings;
  try
    str.SaveToFile(path);
    document.markUnchanged;
  except
    ShowMessage('Cannot store file "'+path+'"!');
  end;
  str.destroy;
end;

procedure TFormMain.addFunction(Sender: TObject);
var
  i : integer;
  el,pel : XML.TElement;
begin
  i := MenuItem7.IndexOf(TMenuItem(Sender));
  if (i < 0) then exit;
  el := XML.TElement.Create(TGuiTab.availlable[i].code.ClassName);
  el.setAttribute('name', TGuiTab.availlable[i].name);
  pel := document.getElementsByTagName('GUITABS', 0);
  if (pel = nil) then begin
    pel := XML.TElement.create('GUITABS');
    document.appendChild(pel);
  end;
  pel.appendChild(el);
  addFunction(el);
end;

procedure TFormMain.addFunction(xmlEl: XML.TElement);
var
  i : integer;
  s : String;
  t : TGuiTab;
label found;
begin
  for i := 0 to TGuiTab.availlableCount-1 do if (UpperCase(TGuiTab.availlable[i].code.ClassName) = xmlEl.tagName) then goto found;
  ShowMessage('Gui Tab Class "'+xmlEl.tagName+'" not found!');
  exit;
  found:
  t := TGuiTab.availlable[i].code.Create(TabControl1, xmlEl);
  t.Parent := TabControl1;
  t.AnchorParallel(akTop, 0, TabControl1);
  t.AnchorParallel(akLeft, 0, TabControl1);
  t.AnchorParallel(akRight, 0, TabControl1);
  t.AnchorParallel(akBottom, 0, TabControl1);
  s := xmlEL.getAttribute('name');
  if (s = '') then s := xmlEl.tagName;
  i := length(tabs);
  setLength(tabs, i+1);
  tabs[i] := t;
  TabControl1.Tabs.Add(s);
  TabControl1.tabIndex := TabControl1.Tabs.count-1;
end;

procedure TFormMain.deleteFunction(Sender: TObject);
var i : integer;
begin
  if (TabControl1.tabIndex < 0) or (TabControl1.tabIndex >= length(tabs)) then exit;
  i := TabControl1.tabIndex;
  tabs[i].destroy;
  for i := i to length(tabs)-2 do tabs[i] := tabs[i+1];
  SetLength(tabs, length(tabs)-1);
  TabControl1.Tabs.Delete(TabControl1.tabIndex);
  selectTab(sender);
end;

procedure TFormMain.selectTab(Sender: TObject);
var i : integer;
begin
  for i := 0 to length(tabs)-1 do tabs[i].visible := i = TabControl1.tabIndex;
end;

procedure TFormMain.closeClick(Sender: TObject);
begin
  close;
end;

procedure TFormMain.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  CloseAction := caFree;
  if (document.changed) then case (MessageDlg('Changes', 'Do you wish to save changes?', mtConfirmation, [mbYes, mbNo, mbAbort], 0)) of
   mrYes : save(sender);
   mrAbort : CloseAction := caNone;
  end;
end;

procedure TFormMain.FormDestroy(Sender: TObject);
begin
  clear(true);
end;

end.

